#include <array>

#include "App.hpp"
#include "Math.hpp"

#include "raylib.h"
#include "raymath.h"

void App::init(int screenWidth, int screenHeight)
{
  this->screenWidth = screenWidth;
  this->screenHeight = screenHeight;
  canvases.assign(2, {});

  for (auto &&canvas : canvases)
  {
    canvas.init(screenWidth);
    canvas.newGen();
  }

  center = {(float)screenWidth / 2, (float)screenHeight / 2};
  float rotation = 0.0f;
  destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  destB = {destA.x, destA.y, (float)screenWidth, (float)screenWidth};
}

void App::update()
{
  if (IsMouseButtonDown(MOUSE_BUTTON_LEFT))
  {
    int mouseX = GetMouseX();
    center.x = mouseX;
    center.y = GetMouseY();
    float l = (float)mouseX / screenWidth;
    rotation = Lerp(45.0f, -45.0f, l);
    destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  }

  if (IsMouseButtonReleased(MOUSE_BUTTON_LEFT))
  {
    canvases[1 - pos].newGen();
    pos = 1 - pos;
    rotation = 0.0f;
    center.x = screenWidth / 2;
    center.y = screenHeight / 2;
    destA = CalculateRect(center, rotation, screenWidth, screenWidth);
  }
}

void App::draw()
{
  ClearBackground(RED);
  canvases[pos].draw(destB, 0.0f);
  canvases[1 - pos].draw(destA, rotation);
}

void App::deinit()
{
  for (size_t i = 0; i < canvases.size(); i++)
  {
    canvases[i].deinit();
  }
}